<?php

namespace Drp\NativeEventApp\GraphQL\Resolvers;

use Drp\NativeEventApp\Auth\Facades\AuthInfoFinder;
use Drp\NativeEventApp\Models\NavigatableItem;
use Drp\NativeEventApp\Models\Navigation;
use Drp\NativeEventApp\Models\Registration;
use GraphQL\Type\Definition\ResolveInfo;
use Nuwave\Lighthouse\Support\Contracts\GraphQLContext;

class NavigatableItemResolver
{
    public function __invoke(Navigation $navigation, array $args, GraphQLContext $context, ResolveInfo $resolveInfo): ?NavigatableItem // phpcs:ignore
    {
        $authenticatable = AuthInfoFinder::current();
        /** @var Registration $registration */
        $registration = Registration::forEventAndAuthenticatable($navigation->event, $authenticatable)->first();

        // Return if navigation item doesn't belong to any segments (always show)
        // OR if the registration belongs to all the segments the navigation imposes, return the item
        if ($navigation->{$resolveInfo->fieldName}->segments->isEmpty()
            || $registration->hasAllSegments($navigation->{$resolveInfo->fieldName}->segments)
        ) {
            return $navigation->{$resolveInfo->fieldName};
        }

        return null;
    }
}
