<?php

namespace Drp\NativeEventApp\Models;

use Drp\NativeEventApp\Models\Concerns\Segmentable;
use Drp\NativeEventApp\Modules\Modules;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Database\Eloquent\Relations\BelongsTo;
use Illuminate\Support\Arr;

/**
 * @property Modules modules
 */
class Page extends Model
{
    use Segmentable;

    /** @var string  */
    protected $table = 'pages';

    public function event(): BelongsTo
    {
        return $this->belongsTo(Event::class, 'event_id');
    }

    public function setModulesAttribute(Modules $modules): void
    {
        $this->attributes['modules'] = $this->castAttributeAsJson('modules', $modules);
    }

    public function getModulesAttribute(): Modules
    {
        return Modules::fromString(Arr::get($this->attributes, 'modules', ''));
    }

    public function scopeEnabled(Builder $query): Builder
    {
        return $query->whereNotNull('enabled_at')->whereNull('enabled_until')
            ->orWhere(static function (Builder $subQuery): Builder {
                return $subQuery->whereDate('enabled_at', '<=', now())
                    ->whereDate('enabled_until', '>=', now());
            });
    }
}
