<?php

namespace Drp\NativeEventApp\Notifications;

use Benwilkins\FCM\FcmMessage;
use Illuminate\Bus\Queueable;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Notifications\Notification;

class PushNotification extends Notification implements ShouldQueue
{
    use Queueable;

    /**
     * @var string
     */
    protected $firebaseDeviceId;

    /**
     * @var string
     */
    protected $senderId;

    public function __construct(string $firebaseDeviceId, string $title, string $message)
    {
        $this->firebaseDeviceId = $firebaseDeviceId;
        $this->title = $title;
        $this->message = $message;
        $this->senderId = config('native-event-app.firebase.fcm_sender_id');
    }

    public function toFcm(): FcmMessage
    {
        return tap($this->getFcmMessage(), function (FcmMessage $message): void {
            $message->content([
                'title' => $this->title,
                'body' => $this->message,
            ]);
        });
    }

    protected function getFcmMessage(): FcmMessage
    {
        return (new FcmMessage())
            ->setHeaders(['project_id' => $this->senderId])
            ->to($this->firebaseDeviceId);
    }

    public function via(): array
    {
        return ['fcm'];
    }
}
