<?php

namespace Drp\NativeEventApp;

use Drp\NativeEventApp\Contracts\Obfuscatable;
use Drp\NativeEventApp\Exceptions\InvalidPhoneNumber;
use Illuminate\Support\Str;

class PhoneNumber implements Obfuscatable
{
    /** @var string */
    private $phoneNumber;

    public function __construct(?string $phoneNumber)
    {
        throw_unless(self::isValid($phoneNumber), InvalidPhoneNumber::invalidFormat($phoneNumber));

        $this->phoneNumber = $phoneNumber;
    }

    /**
     * This will ensure phone numbers are valid UK numbers
     */
    public static function isValid(?string $phoneNumber): bool
    {
        $pattern = '/((\+?44(\s?\(0\)\s?|\s?0\s?|\s)?)|0)7\d{3}(\s)?\d{6}/';

        return (bool) preg_match($pattern, $phoneNumber);
    }

    public function obfuscated(): string
    {
        return substr($this->phoneNumber, '-3', 3);
    }

    public function internationalized(): string
    {
        $phoneNumber = str_replace(['(', ')', ' '], '', $this->phoneNumber);

        if (Str::startsWith($phoneNumber, '+')) {
            $phoneNumber = substr($phoneNumber, 1);
        }

        if (Str::startsWith($phoneNumber, '440')) {
            return Str::replaceFirst('440', '44', $phoneNumber);
        }

        if (Str::startsWith($phoneNumber, '0')) {
            return Str::replaceFirst('0', '44', $phoneNumber);
        }

        return $phoneNumber;
    }

    public function __toString(): string
    {
        return $this->phoneNumber;
    }
}
