<?php

namespace Drp\NativeEventApp\Test\Support\Fakes;

use Drp\NativeEventApp\Models\Quiz;
use Drp\NativeEventApp\Models\QuizQuestion;
use Drp\NativeEventApp\Quiz\Contracts\QuizOperator;
use PHPUnit\Framework\Assert;

class FakeQuizOperator implements QuizOperator
{
    /** @var \Illuminate\Support\Collection */
    protected $created;

    /** @var \Illuminate\Support\Collection */
    protected $started;

    /** @var \Illuminate\Support\Collection */
    protected $ended;

    /** @var \Illuminate\Support\Collection */
    protected $deleted;

    /** @var \Illuminate\Support\Collection */
    protected $startedQuestions;

    /** @var \Illuminate\Support\Collection */
    protected $endedQuestions;

    /** @var \Illuminate\Support\Collection */
    protected $revealed;

    public function __construct()
    {
        $this->created = collect();
        $this->started = collect();
        $this->ended = collect();
        $this->deleted = collect();
        $this->startedQuestions = collect();
        $this->endedQuestions = collect();
        $this->revealed = collect();
    }

    public function create(Quiz $quiz): void
    {
        $this->created->push($quiz);
    }

    public function assertCreateWasTriggered(Quiz $quiz)
    {
        Assert::assertTrue(
            $this->created->contains($quiz),
            "The given quiz (id: $quiz->id, title: $quiz->title) was not registered."
        );
    }

    public function start(Quiz $quiz): void
    {
        $this->started->push($quiz);
    }

    public function assertStartWasTriggered(Quiz $quiz)
    {
        Assert::assertTrue(
            $this->started->contains($quiz),
            "The given quiz (id: $quiz->id, title: $quiz->title) was not started."
        );
    }

    public function end(Quiz $quiz): void
    {
        $this->ended->push($quiz);
    }

    public function assertEndWasTriggered(Quiz $quiz)
    {
        Assert::assertTrue(
            $this->ended->contains($quiz),
            "The given quiz (id: $quiz->id, title: $quiz->title) was not ended."
        );
    }

    public function delete(Quiz $quiz): void
    {
        $this->deleted->push($quiz);
    }

    public function assertDeleteWasTriggered(Quiz $quiz)
    {
        Assert::assertTrue(
            $this->deleted->contains($quiz),
            "The given quiz (id: $quiz->id, title: $quiz->title) was not unregistered."
        );
    }

    public function startQuestion(Quiz $quiz, QuizQuestion $question): void
    {
        $this->startedQuestions->push([
            'quiz' => $quiz,
            'question' => $question,
        ]);
    }

    public function assertStartQuestionWasTriggered(Quiz $quiz, QuizQuestion $question): void
    {
        $result = $this->startedQuestions->contains(function ($item) use ($question, $quiz) {
            return $quiz->is($item['quiz']) && $question->is($item['question']);
        });

        Assert::assertTrue(
            $result,
            "The given quiz (id: $quiz->id, title: $quiz->title) " .
            "and question (id: $question->id, title: $question->title) was not sent to."
        );
    }

    public function endQuestion(Quiz $quiz, QuizQuestion $question): void
    {
        $this->endedQuestions->push([
            'quiz' => $quiz,
            'question' => $question,
        ]);
    }

    public function assertEndQuestionWasTriggered(Quiz $quiz, QuizQuestion $question): void
    {
        $result = $this->endedQuestions->contains(function ($item) use ($question, $quiz) {
            return $quiz->is($item['quiz']) && $question->is($item['question']);
        });

        Assert::assertTrue(
            $result,
            "The given quiz (id: $quiz->id, title: $quiz->title) " .
            "and question (id: $question->id, title: $question->title) was not sent to."
        );
    }

    public function revealAnswer(Quiz $quiz, QuizQuestion $question): void
    {
        $this->revealed->push([
            'quiz' => $quiz,
            'question' => $question,
        ]);
    }

    public function assertRevealAnswerWasTriggered(Quiz $quiz, QuizQuestion $question): void
    {
        $result = $this->revealed->contains(function ($item) use ($question, $quiz) {
            return $quiz->is($item['quiz']) && $question->is($item['question']);
        });

        Assert::assertTrue(
            $result,
            "The given quiz (id: $quiz->id, title: $quiz->title) " .
            "and question (id: $question->id, title: $question->title) was not sent to."
        );
    }
}
