<?php

namespace Drp\NativeEventApp\Test\Unit\Notifications\Listeners;

use Carbon\Carbon;
use Drp\NativeEventApp\Models\Device;
use Drp\NativeEventApp\Models\FailedNotification;
use Drp\NativeEventApp\Models\Notification;
use Drp\NativeEventApp\Models\Registration;
use Drp\NativeEventApp\Notifications\Events\NotificationSendFailed;
use Drp\NativeEventApp\Test\TestCase;
use Exception;
use Illuminate\Contracts\Debug\ExceptionHandler;

class UpdateNotificationOnFailedTest extends TestCase
{
    /** @test */
    public function it_creates_a_failed_notification_entry_when_event_fired(): void
    {
        Carbon::setTestNow('02-04-2020 14:00');

        factory(Registration::class)->create();
        $device = factory(Device::class)->create();
        $exception = new Exception('Notification error message');
        $notification = factory(Notification::class)->state('published')->create();

        $this->mock(ExceptionHandler::class, static function ($mock) use ($exception): void {
            $mock->shouldReceive('report')->with($exception)->once();
        });

        event(new NotificationSendFailed($notification, $device, $exception));

        $this->assertEquals(1, FailedNotification::count());

        $failedNotification = FailedNotification::find(1);
        $this->assertEquals($notification->id, $failedNotification->notification_id);
        $this->assertEquals($device->id, $failedNotification->device_id);
        $this->assertEquals('Notification error message', $failedNotification->error);
    }
}
