<?php

namespace Drp\NativeEventApp\Test\Unit;

use Drp\NativeEventApp\Models\Registration;
use Drp\NativeEventApp\Models\Segment;
use Drp\NativeEventApp\Models\SegmentRegistration;
use Drp\NativeEventApp\Test\TestCase;

class RegistrationTest extends TestCase
{
    /** @test */
    public function join_segments(): void
    {
        $segment1 = factory(Segment::class)->create();
        $segment2 = factory(Segment::class)->create();
        $segments = collect([$segment1, $segment2]);
        $registration = factory(Registration::class)->create();

        $registration->joinSegments($segments);
        $registration->refresh();

        $this->assertCount(2, $registration->segments);
        $this->assertTrue($registration->segments->first()->is($segment1));
        $this->assertTrue($registration->segments->last()->is($segment2));
    }

    /** @test */
    public function wont_join_same_segment_twice(): void
    {
        $segment = factory(Segment::class)->create();
        $registration = factory(Registration::class)->create();

        $registration->joinSegments(collect([$segment]));
        $registration->joinSegments(collect([$segment]));

        $this->assertCount(1, SegmentRegistration::all());
    }

    /** @test */
    public function has_all_segments(): void
    {
        $segment1 = factory(Segment::class)->create();
        $segment2 = factory(Segment::class)->create();
        $segment3 = factory(Segment::class)->create();
        $registration = factory(Registration::class)->create();

        $registration->joinSegments(collect([$segment1, $segment2]));

        $this->assertTrue($registration->hasAllSegments(collect([$segment1, $segment2])));
        $this->assertTrue($registration->hasAllSegments(collect([$segment2])));
        $this->assertFalse($registration->hasAllSegments(collect([$segment2, $segment3])));
    }

    /** @test */
    public function has_at_least_one_segments(): void
    {
        $segment1 = factory(Segment::class)->create();
        $segment2 = factory(Segment::class)->create();
        $segment3 = factory(Segment::class)->create();
        $registration = factory(Registration::class)->create();

        $registration->joinSegments(collect([$segment1, $segment2]));

        $this->assertTrue($registration->hasAtLeastOneSegment(collect([$segment1, $segment2])));
        $this->assertTrue($registration->hasAtLeastOneSegment(collect([$segment2])));
        $this->assertTrue($registration->hasAtLeastOneSegment(collect([$segment2, $segment3])));
        $this->assertFalse($registration->hasAtLeastOneSegment(collect([$segment3])));
    }
}
